// vimeo.js, 2016
// @author ED

var antvd = (function(antvd)
{
    if ( ! antvd.AntLib )
    {
        antvd.AntLib = AntLib;
    }
    
    const Ci = Components.interfaces;
    const Cc = Components.classes;
    

    Components.utils.import("resource://gre/modules/NetUtil.jsm");
    Components.utils.import("resource://gre/modules/Downloads.jsm");
    Components.utils.import("resource://gre/modules/FileUtils.jsm");
    Components.utils.import("resource://gre/modules/Services.jsm");

    // The formula of URL of each segment is the following.
    // There is URL of master.json file. Inside JSON file, there is field base_url which is basic for all videos
    // described in master.json. This field should be added to master.json URL which will lead to its stripping
    // because base_url is relative url (e.g. '../../')
    // For each video, there is also base_url field, which should be added to master.json URL. At the end
    // filename of segment should be added.
    // So, the end segment URL formula looks like this:
    //
    // Segment URI = ((Master JSON URI + Base URL) + Video Base URL) + Segment Filename
    //
    // Since Base URL is relative URL, from math point of view this is the same as -(Base URL), so it is like
    // adding a negative number
    //
    function VimeoMediaRequest(Document, MasterJsonUrl, ClipId, BaseURL, VimeoVideoJSON, VimeoAudioJSON)
    {
        this._document = Document;
        this.m_ClipId = ClipId;
		
		this.m_VideoStream = {
            init_segment: null,
            segments: [],
            segment_base_uri: "",
            content_type: ""
        };
		
		this.m_AudioStream = {
            init_segment: null,
            segments: [],
            segment_base_uri: "",
            content_type: ""
        }
        
        // Strip trailing filename from master JSON
        let _segment_base_uri = MasterJsonUrl;
        
        let lastSlashIndex = MasterJsonUrl.lastIndexOf("/");
        
        if (lastSlashIndex != -1)
        {
            _segment_base_uri = MasterJsonUrl.substring(0, lastSlashIndex);
        }
        
        // Master JSON URI + Base URL
        this.m_VideoStream.segment_base_uri = antvd.AntLib.concatAndResolveUrl(_segment_base_uri, BaseURL);
        
        // (Master JSON URI + Base URL) + Video Base URL
        this.m_VideoStream.segment_base_uri = this.m_VideoStream.segment_base_uri + VimeoVideoJSON['base_url'];
        
        // Check for trailing slash
        if(this.m_VideoStream.segment_base_uri.endsWith("/") == false)
        {
            this.m_VideoStream.segment_base_uri = this.m_VideoStream.segment_base_uri + "/";
        }
        
        // Segment filename is added in VimeoMediaRequest.download()
                
        // Extract data from JSON description of Vimeo video
        this.m_VideoStream.init_segment = VimeoVideoJSON['init_segment'];
        this.m_VideoStream.segments = VimeoVideoJSON['segments'];
        this.m_VideoStream.content_type = VimeoVideoJSON['mime_type'];
        
        // Copy audio stream object
        if (VimeoAudioJSON != null)
        {
			this.m_AudioStream.segment_base_uri = antvd.AntLib.concatAndResolveUrl(_segment_base_uri, BaseURL);
			
			this.m_AudioStream.segment_base_uri = this.m_AudioStream.segment_base_uri + VimeoAudioJSON['base_url'];

            // Check for trailing slash
            if(this.m_AudioStream.segment_base_uri.endsWith("/") == false)
            {
                this.m_AudioStream.segment_base_uri = this.m_AudioStream.segment_base_uri + "/";
            }
            
            this.m_AudioStream.init_segment = VimeoAudioJSON['init_segment'];
            this.m_AudioStream.segments = VimeoAudioJSON['segments'];
            this.m_AudioStream.content_type = VimeoAudioJSON['mime_type'];
		}

        // Create clip name that consists of video resolution and clean document title
        // (e.g. "1920x1080 In Japan - Vimeo")
        this.m_CleanDocumentName = antvd.AntLib.sanitize(this._document.title).replace(/[,:()\[\]\|"'.`~▶]/ig,"").trim();
        let cleanName = antvd.AntLib.sprintf('%sx%s %s', VimeoVideoJSON['width'], VimeoVideoJSON['height'], this.m_CleanDocumentName);
        
        this._base = new antvd.MediaRequest(Document.documentURIObject, Document.referrer, cleanName, 0);
    };
    
    VimeoMediaRequest.prototype =
    {
        _base: null,
        _document: null,
        
        m_ClipId: null,             // Id of clip
        m_CleanDocumentName: null,  // Document title, stripped out of extra characters, that can be a filename        
        m_VideoStream: null,        
        m_AudioStream: null,

        get displayName()
        {
            return this._base.displayName;
        },
        
        get size()
        {
            return this._base.size;
        },
    
        compare: function(request)
        {
        },
        
        getFileName: function()
        {
            let extension = ".bin";
            
            switch (this.m_VideoStream.content_type.toLowerCase())
            {
                case "video/mp4":
                    extension = "mp4";
                    break;
                
                case "video/webm":
                    extension = "webm";
                    break;
            }
            
            return this.m_CleanDocumentName + "." + extension;
        },
        
        download: function(library)
        {
            let ctx = this;
            let executor = null;
            let converterConf = antvd.ConverterPackage.getDefault();
            
            try
            {
                library.ensureConfigured();
                converterConf.ensureConfigured();
            }
            catch (ex)
            {
                return Promise.reject(ex);
            }
            
            // No audio stream, only video (contains embedded audio track)
            var _downloadVideoFunction = function()
            {
                var initSegmentArray;           // array[]
                var outputFile;                 // nsIFile
                var outputFilePTU;              // nsIFile
                var outputStream;               // nsIFileOutputStream 
                var binaryOutputStream;         // nsIBinaryOutputStream
                
                let converter = new antvd.Converter(converterConf);

                // Convert init segment (Base64 string) into Uint8 array and
                // write it to temporary file
                initSegmentArray = antvd.AntLib.convertDataURIToBinary(ctx.m_VideoStream.init_segment);
                
                if (initSegmentArray.length == 0)
                {
                    antvd.AntLib.logError(
                        "VimeoMediaRequest.download (vimeo.js)",
                        "Initialization segment array is empty", null
                    );
                    
                    return;
                }
                
                // Open output file, stream, and binary stream
                // First write initialization header into output file
                // Then download and write all chunks
                
                outputFile = FileUtils.getFile("TmpD", ["output-" + ctx.m_CleanDocumentName]);
                outputFilePTU = FileUtils.getFile("TmpD", ["output-ptu-" + ctx.m_CleanDocumentName]);
                
                outputStream = FileUtils.openSafeFileOutputStream(outputFile, FileUtils.MODE_WRONLY | FileUtils.MODE_CREATE);
                
                binaryOutputStream = Components.classes["@mozilla.org/binaryoutputstream;1"].createInstance(Components.interfaces.nsIBinaryOutputStream);
                
                binaryOutputStream.setOutputStream(outputStream);
                binaryOutputStream.writeByteArray(initSegmentArray, initSegmentArray.length);
                
                // Download rest of segments
                try
                {
                    for(let i = 0; i < ctx.m_SegmentsArray.length; i++)
                    {
                        let chunkFile, chunkStream, binaryChunkStream;
                        let chunkBinaryContent = [];
                        let _segment_url = NetUtil.newURI(ctx.m_VideoStream.segment_base_uri + ctx.m_VideoStream.segments[i]['url']);
                        
                        let _video = library.download(
                            _segment_url,
                            antvd.AntLib.sprintf("Chunk-%d-of-%d-%s", i+1, ctx.m_VideoStream.segments.length, ctx.m_CleanDocumentName),
                            true
                        );
                        
                        _svideo = yield _video;
                        
                        // Open chunk as binary stream
                        chunkFile = Components.classes["@mozilla.org/file/local;1"].createInstance(
                            Components.interfaces.nsILocalFile
                        );

                        chunkStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(
                            Components.interfaces.nsIFileInputStream
                        );
                        
                        binaryChunkStream = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(
                            Components.interfaces.nsIBinaryInputStream
                        );
                        
                        chunkFile.initWithPath(_svideo.target);

                        chunkStream.init(chunkFile, -1, 0, 0);
                        
                        binaryChunkStream.setInputStream(chunkStream);
                        
                        chunkBinaryContent = binaryChunkStream.readByteArray(chunkFile.fileSize);
                        
                        // Copy chunk stream into output binary stream
                        binaryOutputStream.writeByteArray(chunkBinaryContent, chunkBinaryContent.length);
                        
                        binaryOutputStream.flush();
                        
                        chunkStream.close();
                    }
                }
                catch (ex)
                {
                    antvd.AntLib.logError("VimeoMediaRequest.download (vimeo.js)", "Failed to download streams", ex);
    
                    throw ex;
                }
    
                FileUtils.closeSafeFileOutputStream(outputStream);

                // Pass video through media converter
                converter.setName(ctx._base.displayName);
    
                try
                {
                    yield converter.passthrough(outputFile, outputFilePTU, ['-f', 'mp4']);
                }
                catch (ex)
                {
                    antvd.AntLib.logError("VimeoMediaRequest.download (hls.js)", "Failed to pass-through media", ex);

                    throw ex;
                }

                try
                {
                    yield library.save(
                        {
                            uri: Services.io.newFileURI(outputFilePTU),
                            filename: ctx.getFileName(),
                            origin:
                            {
                                url: ctx._base._originUrl,
                                title: ctx.m_CleanDocumentName,
                                domain: antvd.AntLib.getDomain(ctx._base._originUrl)
                            }
                        }
                    );
                }
                finally
                {
                    try
                    {
                        // Remove init segment + rest segments
                        //for(let i = 0; i < _svideos.length; i++)
                        //{
                        //    FileUtils.File(_svideos[i].target).remove(false);
                        //}
                    }
                    catch (_e0)
                    {
                        antvd.AntLib.logWarning(
                            "VimeoMediaRequest.download (vimeo.js)", "Failed to cleanup temporary files", _e0
                        );
                    }
                }
            };
            
            // Contains separate audio and video streams
            var _downloadVideoAndAudioFunction = function()
            {
                var initSegmentArray;           // array[]
                var video_outputFile;           // nsIFile
                var video_outputStream;         // nsIFileOutputStream 
                var audio_outputFile;           // nsIFile
                var audio_outputStream;         // nsIFileOutputStream 
                var binaryOutputStream;         // nsIBinaryOutputStream
				var cleanupChunks = [];
                
                if (ctx.m_AudioStream == null)
                {
                    antvd.AntLib.logError(
                        "VimeoMediaRequest.download (vimeo.js)",
                        "Audio stream object is null, unable to proceed", null
                    );
                    
                    return;
				}
                
                let converter = new antvd.Converter(converterConf);
                
                // Process video chunks
                //
                //
                
                // Convert init segment (Base64 string) into Uint8 array and
                // write it to temporary file
                initSegmentArray = antvd.AntLib.convertDataURIToBinary(ctx.m_VideoStream.init_segment);
                
                if (initSegmentArray.length == 0)
                {
                    antvd.AntLib.logError(
                        "VimeoMediaRequest.download (vimeo.js)",
                        "Video initialization segment array is empty", null
                    );
                    
                    return;
                }
                else
                {
                    // Open output file, stream, and binary stream
                    // First write initialization header into output file
                    // Then download and write all chunks
                    
                    video_outputFile = FileUtils.getFile("TmpD", ["output-video-" + ctx.m_CleanDocumentName]);
					cleanupChunks.push(video_outputFile.path);

                    video_outputStream = FileUtils.openSafeFileOutputStream(video_outputFile, FileUtils.MODE_WRONLY | FileUtils.MODE_CREATE);
                    
                    binaryOutputStream = Components.classes["@mozilla.org/binaryoutputstream;1"].createInstance(Components.interfaces.nsIBinaryOutputStream);
                    
                    binaryOutputStream.setOutputStream(video_outputStream);
                    binaryOutputStream.writeByteArray(initSegmentArray, initSegmentArray.length);
                }
                
                // Download rest of segments
                try
                {
                    for(let i = 0; i < ctx.m_VideoStream.segments.length; i++)
                    {
                        let chunkFile, chunkStream, binaryChunkStream;
                        let chunkBinaryContent = [];
                        let _segment_url = NetUtil.newURI(ctx.m_VideoStream.segment_base_uri + ctx.m_VideoStream.segments[i]['url']);
                        
                        let _video = library.download(
                            _segment_url,
                            antvd.AntLib.sprintf("Chunk-Video-%d-of-%d-%s", i+1, ctx.m_VideoStream.segments.length, ctx.m_CleanDocumentName),
                            true
                        );
                        
                        _svideo = yield _video;
                        
                        // Open chunk as binary stream
                        chunkFile = Components.classes["@mozilla.org/file/local;1"].createInstance(
                            Components.interfaces.nsILocalFile
                        );

                        chunkStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(
                            Components.interfaces.nsIFileInputStream
                        );
                        
                        binaryChunkStream = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(
                            Components.interfaces.nsIBinaryInputStream
                        );
                        
                        chunkFile.initWithPath(_svideo.target);
						cleanupChunks.push(_svideo.target);

                        chunkStream.init(chunkFile, -1, 0, 0);
                        
                        binaryChunkStream.setInputStream(chunkStream);
                        
                        chunkBinaryContent = binaryChunkStream.readByteArray(chunkFile.fileSize);
                        
                        // Copy chunk stream into output binary stream
                        binaryOutputStream.writeByteArray(chunkBinaryContent, chunkBinaryContent.length);
                        
                        binaryOutputStream.flush();
                        
                        chunkStream.close();
                    }
                }
                catch (ex)
                {
                    antvd.AntLib.logError("VimeoMediaRequest.download (vimeo.js)", "Failed to download streams", ex);
    
                    throw ex;
                }
    
                FileUtils.closeSafeFileOutputStream(video_outputStream);

                
                // Process audio chunks
                //
                //
                initSegmentArray = antvd.AntLib.convertDataURIToBinary(ctx.m_AudioStream.init_segment);
                
                if (initSegmentArray.length == 0)
                {
                    antvd.AntLib.logError(
                        "VimeoMediaRequest.download (vimeo.js)",
                        "Audio initialization segment array is empty", null
                    );
                    
                    return;
                }
                else
                {
                    // Open output file, stream, and binary stream
                    // First write initialization header into output file
                    // Then download and write all chunks                    
                    audio_outputFile = FileUtils.getFile("TmpD", ["output-audio-" + ctx.m_CleanDocumentName]);
					cleanupChunks.push(audio_outputFile.path);
                    
                    audio_outputStream = FileUtils.openSafeFileOutputStream(audio_outputFile, FileUtils.MODE_WRONLY | FileUtils.MODE_CREATE);
                    
                    binaryOutputStream = Components.classes["@mozilla.org/binaryoutputstream;1"].createInstance(Components.interfaces.nsIBinaryOutputStream);
                    
                    binaryOutputStream.setOutputStream(audio_outputStream);
                    binaryOutputStream.writeByteArray(initSegmentArray, initSegmentArray.length);
                }
                
                // Download rest of segments
                try
                {
                    for(let i = 0; i < ctx.m_AudioStream.segments.length; i++)
                    {
                        let chunkFile, chunkStream, binaryChunkStream;
                        let chunkBinaryContent = [];
                        let _segment_url = NetUtil.newURI(ctx.m_AudioStream.segment_base_uri + ctx.m_AudioStream.segments[i]['url']);
                        
                        let _video = library.download(
                            _segment_url,
                            antvd.AntLib.sprintf("Chunk-Audio-%d-of-%d-%s", i+1, ctx.m_AudioStream.segments.length, ctx.m_CleanDocumentName),
                            true
                        );
                        
                        _svideo = yield _video;
                        
                        // Open chunk as binary stream
                        chunkFile = Components.classes["@mozilla.org/file/local;1"].createInstance(
                            Components.interfaces.nsILocalFile
                        );

                        chunkStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(
                            Components.interfaces.nsIFileInputStream
                        );
                        
                        binaryChunkStream = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(
                            Components.interfaces.nsIBinaryInputStream
                        );
                        
                        chunkFile.initWithPath(_svideo.target);
						cleanupChunks.push(_svideo.target);

                        chunkStream.init(chunkFile, -1, 0, 0);
                        
                        binaryChunkStream.setInputStream(chunkStream);
                        
                        chunkBinaryContent = binaryChunkStream.readByteArray(chunkFile.fileSize);
                        
                        // Copy chunk stream into output binary stream
                        binaryOutputStream.writeByteArray(chunkBinaryContent, chunkBinaryContent.length);
                        
                        binaryOutputStream.flush();
                        
                        chunkStream.close();
                    }
                }
                catch (ex)
                {
                    antvd.AntLib.logError("VimeoMediaRequest.download (vimeo.js)", "Failed to download streams", ex);
    
                    throw ex;
                }
    
                FileUtils.closeSafeFileOutputStream(audio_outputStream);

                // Join audio and video chunks
                //
                converter.setName(ctx._base.displayName);
    
                try
                {
                    yield converter.join(
                        video_outputFile.path,
                        audio_outputFile.path,
                        ctx.m_VideoStream.content_type,
                        ctx.m_AudioStream.content_type
                    );
                }
                catch (ex)
                {
                    antvd.AntLib.logError(
                        "VimeoMediaRequest.download (hls.js)",
                        "Failed to merge streams", ex
                    );

                    throw ex;
                }

                try
                {
                    yield library.save(
                        {
                            uri: converter.getUri(),
                            filename: ctx.getFileName(),
                            origin:
                            {
                                url: ctx._base._originUrl,
                                title: ctx.m_CleanDocumentName,
                                domain: antvd.AntLib.getDomain(ctx._base._originUrl)
                            }
                        }
                    );
                }
                finally
                {
					// Clean up temporary files
					if (cleanupChunks.length > 0)
					{
						for(let i = 0; i < cleanupChunks.length; i++)
						{
							try
							{
								FileUtils.File(cleanupChunks[i]).remove(false);
							}
							catch (_e0)
							{
								antvd.AntLib.logWarning(
									"VimeoMediaRequest.download (vimeo.js)",
									"Failed to cleanup temporary file " + cleanupChunks[i], _e0
								);
							}
						}
                    }
                }
            };
            
            if (ctx.m_AudioStream == null)
            {
				executor = _downloadVideoFunction;
			}
            else
            {
                executor = _downloadVideoAndAudioFunction;
            }
            
            return Task.spawn(executor);
        },

        reportDownload: function()
        {
            // return this._base.reportDownload();
        },
    
        release: function()
        {
        }
    };

    function VimeoSearchResult(MasterJSONUri, Document)
    {
        var m_Ctx = this;
        
        var m_MasterJSONUri = MasterJSONUri;
        var m_Document = Document;
        var m_Callback = null;
        
        const Ci = Components.interfaces;
        const Cc = Components.classes;
    
        const ID_IOSERVICE_CONTRACT = "@mozilla.org/network/io-service;1";
    
        // ISearchResult implementation
        
        // Asynchronously downloads and parses the manifest
        // @member asyncFetch
        // @param {Function} clbck May be called multiple times.
        //                         An instance of FlvLink is as a single argument
        // @returns {undefined} nothing
        this.asyncFetch = function(callback)
        {
            m_Callback = callback;
            
            this.withContentUri(m_MasterJSONUri, this.processMasterJSON);
        };
        
        // processMasterJSON method
        // Synchronously parses the content and builds a valid object of VideoSource
        // @member processMasterJSON
        // @param {String} content Vimeo master JSON textual content
        this.processMasterJSON = function(content)
        {
            // type Object
            let masterJsonContent = null;
            let clipId = null;
            let baseUrl = null;
            let videosArray = [];
            let audiosArray = [];
            let selectedAudioStream = null;
            let _output = "";
    
            try
            {
                masterJsonContent = JSON.parse(content);
            
                clipId = masterJsonContent['clip_id'];
                baseUrl = masterJsonContent['base_url'];
                videosArray = masterJsonContent['video'];
                
                if (masterJsonContent.hasOwnProperty('audio'))
                {
					audiosArray = masterJsonContent['audio'];
				}
                
                antvd.AntLib.toLog(
                    "VimeoSearchResult.processMasterJSON (vimeo.js)",
                    antvd.AntLib.sprintf(
                        "Clip ID: %s; Base URL: %s; Video streams: %d; Audio streams: %d",
                        clipId, baseUrl, videosArray.length, audiosArray.length
                    )
                );
                
                // Process audio streams and pick the average one
                //
                if (audiosArray.length > 0)
                {
                    for(var a in audiosArray)
                    {
                        var audio = audiosArray[a];
                        
                        _output = antvd.AntLib.sprintf(
                            '   Audio #%d, type %s, bitrate %s, channels %d, sample rate %d, segments count %d',
                            a,
                            audio['mime_type'], audio['avg_bitrate'], audio['channels'],
                            audio['sample_rate'], audio['segments'].length
                        );
                        
                        antvd.AntLib.toLog("VimeoSearchResult.processMasterJSON (vimeo.js)", _output);
                    }
                    
                    if (audiosArray.length <= 2)
                    {
						selectedAudioStream = audiosArray[0];
					}
                    else
                    {
						selectedAudioStream = audiosArray[audiosArray.length / 2];
					}
                    
                    _output = antvd.AntLib.sprintf(
                        '   Picked up audio stream: type %s, bitrate %s, channels %d, sample rate %d, segments count %d',
                        selectedAudioStream['mime_type'], selectedAudioStream['avg_bitrate'], selectedAudioStream['channels'],
                        selectedAudioStream['sample_rate'], selectedAudioStream['segments'].length
                    );
                    
                    antvd.AntLib.toLog("VimeoSearchResult.processMasterJSON (vimeo.js)", _output);
				}
                else
                {
                    antvd.AntLib.toLog(
                        "VimeoSearchResult.processMasterJSON (vimeo.js)",
                        "There are no separate audio streams in the manifest"
                    );
                }
                
                // Process video streams. For each video stream, a previously defined
                // audio stream will be used.
                // If no audio stream was detected, then video stream already contains
                // audio track
                //
                if(videosArray.length > 0)
                {
                    for (var v in videosArray)
                    {
                        var video = videosArray[v];
                        
                        _output = antvd.AntLib.sprintf(
                            '   Video #%d, resolution %sx%s, format %s, segments count %d',
                            v,
                            video['width'], video['height'],
                            video['mime_type'],
                            video['segments'].length
                        );
                        
                        antvd.AntLib.toLog("VimeoSearchResult.processMasterJSON (vimeo.js)", _output);

                        let vimeoMediaRequest = new VimeoMediaRequest(
                            m_Document,
                            m_MasterJSONUri.spec,
                            clipId, baseUrl,
                            video, selectedAudioStream
                        );
                        
                        m_Callback(vimeoMediaRequest);
                    }
                }
            }
            catch (ex)
            {
                antvd.AntLib.logError(
                    "VimeoSearchResult.processMasterJSON (vimeo.js)",
                    "Failed to parse master JSON " + m_MasterJSONUri.spec,
                    ex
                );
                
                return;
            }            
        };
        
        // withContentUri private method
        // @private
        // @member withContentUri
        // @param {nsIURI} uri Uri of the remote resource
        // @param {Function} func Function to be supplied with content of the resource
        // @param {Function} [err=null] Function to be called in case of failure
        this.withContentUri = function(uri, func, err)
        {
            let hr = new XMLHttpRequest();
            
            try
            {
                hr.onload = function(ev)
                {
                    if (hr.status == 200)
                    {
                        func(hr.responseText);
                    }
                    else
                    {
                        antvd.AntLib.toLog(
                            "VimeoSearchResult.withContentUri (vimeo.js)",
                            "Failed to fetch content: " + "\n   URI: " + uri.spec +
                            "\n   Error: " + hr.statusText +
                            "\n   Status: " + hr.status
                        );
                    }
                };
            
                hr.open("GET", uri.spec, true);
                hr.send();
            }
            catch(e)
            {
                antvd.AntLib.logError(
                    "VimeoSearchResult.withContentUri (vimeo.js)",
                    "Async HTTP request failed, URI: " + uri.spec,
                    e
                );
            }
        };
        
        // @member uriFromString
        // @param {String} spec
        // @returns {nsIURI}
        this.uriFromString = function(spec)
        {
            var ioService = Cc[ID_IOSERVICE_CONTRACT].getService(Ci.nsIIOService);
            return ioService.newURI(spec, null, null);
        };
    };

    // @class VimeoSearchStrategy
    // @implements ISearchStrategy
   antvd.VimeoSearchStrategy = function()
   {
       var ctx = this;
   
       const reManifestPathEnd = "/master.json";
       const reManifestType = "application/json";
   
       // ISearchStrategy implementation
   
       // @member isApplicable
       // @param {Document} document
       // @param {nsIHttpChannel} channel
       // @returns {Boolean}
       this.isApplicable = function(document, channel)
       {
            var docUri = document.documentURIObject;
            var reqUri = channel.URI;
            var host = null;
            
            if (docUri.spec.indexOf(reManifestPathEnd) != -1)
            {
                host = docUri;
            }
            else if (reqUri.spec.indexOf(reManifestPathEnd) != -1)
            {
                host = reqUri;
            }
            
            if (host != null)
            {
                antvd.AntLib.toLog(
                    "VimeoSearchStrategy.isApplicable (vimeo.js)",
                    antvd.AntLib.sprintf(
                        "Found manifest file %s at host %s, full URI %s",
                        reManifestPathEnd, host.host, host.spec
                    )
                );
            }
   
           return (host != null) ? true : false;
       };
   
       // @member search
       // @param {Document} document Owning document
       // @param {nsIHttpChannel} channel Request's channel to analyze
       // @param {Function} found The function 'found' is to be called in case if video
       //                         content is found. It may be invoked multiple times.
       //                         The single argument is `flvLink:AntFlvLink
       // @returns {undefined} nothing
       this.search = function(document, channel, found)
       {
            if (!document || !channel || !found)
            {
                return;
            }
            
            var uri = channel.URI;
            
            if ( uri.path.search(reManifestPathEnd) != -1 && channel.contentType == reManifestType)
            {
                var searchResult = new VimeoSearchResult(uri, document);
                
                searchResult.asyncFetch(found);
            }
       };
   };
   
   return antvd;

})(antvd);
